/*! \file warpmac.c
\brief This framework allows for custom MAC implementations on WARP.

@version 3.0 (SVN rev 824)
@author Chris Hunter

This code is a collection of high-level and low-level functions to enable custom MAC layers on warp.
At the highest level, these functions provide many MAC commonalities like carrier sensing and random
exponential backoffs. At the lowest level, these functions wrap around hardware drivers to provide
give user code access to features of the PHY, radio controller, and many other peripherals.
*/

#include "xparameters.h"
#include "xemac.h"
#include "xemac_l.h"
#include "xstatus.h"
#include "errno.h"
#include "xexception_l.h"
#include "stddef.h"
#include <stdio.h>
#include <string.h>

//Header files for Xilinx peripheral drivers
#include "xintc.h"
#include "xgpio.h"

//Header files for WARP libraries
#include "warpmac.h"

//Header files for WARP peripheral drivers
#include "ofdm_AGC_mimo.h"
#include "ofdm_pktDetector_mimo.h"
#include "ofdm_timer.h"
#include "ofdm_txrx_mimo.h"
#include "EEPROM.h"
#include "radio_controller_basic.h"
#include "radio_controller_ext.h"
#include "radio_controller_adv.h"

//Header for WARPPHY Interface
#include "warpphy.h"

///Instantiates the Maccontrol control structure
Maccontrol controlStruct;

///Instantiates the general-purpose input-output peripheral driver
XGpio Gpio;

///Instantiates the Xilinx Ethernet MAC peripheral driver
XEmac Emac;

///Instantiates the Xilinx interrupt controller peripheral driver
XIntc Intc;

///Points to a user-provided Macframe where parsed header information will be copied to
Macframe* rxPacket;
///Points to a user-provided Macframe where parsed header information will be copied from
Macframe* txPacket;

///"Low" LED states
unsigned int ledStatesLow[2] = {1,2};
///Current index into "low" LED states
unsigned char ledStatesIndexLow = 0;
///"High" LED states
unsigned int ledStatesHigh[2] = {4,8};
///Current index into "high" LED states
unsigned char ledStatesIndexHigh = 0;


//These the addresses pointed to by these variables are set by the
//user via the callback registration functions
///Pointer to user's handler
void (*usr_upbutton) ();
///Pointer to user's handler
void (*usr_leftbutton) ();
///Pointer to user's handler
void (*usr_middlebutton) ();
///Pointer to user's handler
void (*usr_rightbutton) ();
///Pointer to user's handler
void (*usr_badPacketHandler) ();
///Pointer to user's handler
void (*usr_goodPacketHandler) ();
///Pointer to user's handler
void (*usr_goodHeaderHandler) ();
///Pointer to user's handler
void (*usr_txDoneHandler) ();
///Pointer to user's handler
void (*usr_timerHandler) ();
///Pointer to user's handler
void (*usr_emacHandler) ();
///Null handler. Catches calls if user failed to attach callbacks.
void nullHandler(void* param){};




///@brief Initializes the framework and all hardware peripherals.
///
///This function sets reasonable default values for many of the parameters of the MAC, configures
///interrupts and exceptions, configures Ethernet, and finally initializes the custom peripherals
///such as the radio controller, the PHY, the packet detector, and the automatic gain control block.
int warpmac_init() {
	usr_upbutton = nullHandler;
	usr_leftbutton = nullHandler;
	usr_middlebutton = nullHandler;
	usr_rightbutton = nullHandler;
	usr_badPacketHandler = nullHandler;
	usr_goodPacketHandler = nullHandler;
	usr_goodHeaderHandler = nullHandler;
	usr_txDoneHandler = nullHandler;
	
	xil_printf("Initializing WARPMAC...\r\n");

	/////////////////////////MAC Parameters///////////////////////
	xil_printf("	Initializing controlStruct...");
	controlStruct.maxReSend = 4;
	controlStruct.currBackoff = 0;
	controlStruct.timerEnabled = 0;
	controlStruct.maxBackoff = 5;
	controlStruct.txBuffIndex = 0;
	controlStruct.rxBuffIndex = 2;
	warpphy_setBuffs(controlStruct.txBuffIndex, controlStruct.rxBuffIndex);
	xil_printf("complete!\r\n");
	//////////////////////////////////////////////////////////////

	/////////////////////////LEDs///////////////////////
	xil_printf("	Initializing LEDs...");
	//Turns on alternating LEDs
	XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x0005);
	xil_printf("complete!\r\n");
	////////////////////////////////////////////////////

	/////////////////////////Interrupts///////////////////////
	xil_printf("	Initializing Interrupts...");
	// Initialize the push buttons
	XGpio_Initialize(&Gpio, XPAR_PUSH_BUTTONS_4BIT_DEVICE_ID);
	XGpio_mSetDataDirection(XPAR_PUSH_BUTTONS_4BIT_BASEADDR, 1, 0x3);
	XGpio_InterruptEnable(&Gpio, XGPIO_IR_CH1_MASK);
	XGpio_InterruptGlobalEnable(&Gpio);

	//Initialize the interrupt controller
	XIntc_Initialize(&Intc, XPAR_OPB_INTC_0_DEVICE_ID);
	XIntc_SetIntrSvcOption(XPAR_OPB_INTC_0_BASEADDR,XIN_SVC_ALL_ISRS_OPTION);
	//Connect the interrupt controller to the interrupt handlers
	XIntc_Connect(&Intc, XPAR_OPB_INTC_0_PUSH_BUTTONS_4BIT_IP2INTC_IRPT_INTR,(XInterruptHandler)pb_int_handler, NULL);
	XIntc_Connect(&Intc, XPAR_OPB_INTC_0_OFDM_TXRX_MIMO_OPBW_0_RX_INT_GOODPKT_INTR,(XInterruptHandler)rxPhyGood, NULL);
	XIntc_Connect(&Intc, XPAR_OPB_INTC_0_OFDM_TXRX_MIMO_OPBW_0_RX_INT_BADPKT_INTR,(XInterruptHandler)rxPhyBad, NULL);
	XIntc_Connect(&Intc, XPAR_OPB_INTC_0_OFDM_TXRX_MIMO_OPBW_0_RX_INT_GOODHEADER_INTR,(XInterruptHandler)rxPhyGoodHeader, NULL);
	XIntc_Connect(&Intc, XPAR_OPB_INTC_0_OFDM_TXRX_MIMO_OPBW_0_TX_INT_PKTDONE_INTR,(XInterruptHandler)txPhyPktDone, NULL);
	XIntc_Connect(&Intc, XPAR_OPB_INTC_0_OFDM_TIMER_OPBW_0_TIMEREXPIRE_INTR,(XInterruptHandler)timer_int_handler, NULL);
	XIntc_Connect(&Intc, XPAR_OPB_INTC_0_ETHERNET_MAC_IP2INTC_IRPT_INTR,(XInterruptHandler) XEmac_IntrHandlerFifo, &Emac);

    //Start the interrupt controller and enable the interrupts
	XIntc_Start(&Intc, XIN_REAL_MODE);
	XIntc_Enable(&Intc, XPAR_OPB_INTC_0_PUSH_BUTTONS_4BIT_IP2INTC_IRPT_INTR);
	XIntc_Enable(&Intc, XPAR_OPB_INTC_0_OFDM_TXRX_MIMO_OPBW_0_RX_INT_GOODHEADER_INTR);
	//XIntc_Enable(&Intc, XPAR_OPB_INTC_0_OFDM_TXRX_MIMO_OPBW_0_TX_INT_PKTDONE_INTR);
	XIntc_Enable(&Intc, XPAR_OPB_INTC_0_OFDM_TXRX_MIMO_OPBW_0_RX_INT_GOODPKT_INTR);
	XIntc_Enable(&Intc, XPAR_OPB_INTC_0_OFDM_TXRX_MIMO_OPBW_0_RX_INT_BADPKT_INTR);
	XIntc_Enable(&Intc, XPAR_OPB_INTC_0_OFDM_TIMER_OPBW_0_TIMEREXPIRE_INTR);
	XIntc_Enable(&Intc, XPAR_OPB_INTC_0_ETHERNET_MAC_IP2INTC_IRPT_INTR);
	xil_printf("complete!\r\n");
	//////////////////////////////////////////////////////////

	/////////////////////////Exceptions///////////////////////
	xil_printf("	Enabling Exceptions...");
	XExc_Init();
	XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT,(XExceptionHandler)XIntc_InterruptHandler,&Intc);
	XExc_mEnableExceptions(XEXC_NON_CRITICAL);
	xil_printf("complete!\r\n");
	//////////////////////////////////////////////////////////

	/////////////////////////Ethernet///////////////////////
	xil_printf("	Initializing Ethernet...");
	XStatus Status;
	Xuint32 Options;
	Xuint16 DeviceId = XPAR_ETHERNET_MAC_DEVICE_ID;

	//Reset the Ethernet PHY
	XEmac_mPhyReset(XPAR_ETHERNET_MAC_BASEADDR);

	//Initialize the XEmac component. The device ID chosen should be
	//configured without DMA (see the xemac_g.c file)
    Status = XEmac_Initialize(&Emac, DeviceId);
    if (Status != XST_SUCCESS)
    {
	 	xil_printf("Failed to initialize EMAC\r\n");
        return XST_FAILURE;
    }
	xil_printf("\tinit succeeded\r\n");

/* Commenting out - the selfTest never returns on a soft reset!
	//Run self-test on the device, which verifies basic sanity of the
	//device and the driver.
    Status = XEmac_SelfTest(&Emac);
    if (Status != XST_SUCCESS)
    {
		xil_printf("Self Test Failed\r\n");
        return XST_FAILURE;
    }
	xil_printf("\tselfTest succeeded\r\n");
*/

	//Configure the device to be promiscuous
    Options = (XEM_INSERT_FCS_OPTION |
			   XEM_INSERT_PAD_OPTION |
			   XEM_UNICAST_OPTION |
			   XEM_BROADCAST_OPTION |
			   XEM_PROMISC_OPTION|
			   XEM_STRIP_PAD_FCS_OPTION);

    Status = XEmac_SetOptions(&Emac, Options);
    if (Status != XST_SUCCESS)
    {
	 xil_printf("Failed to set options of EMAC\r\n");
        return XST_FAILURE;
    }
	xil_printf("\tsetOptions succeeded\r\n");

	//Start the device
	XEmac_SetFifoSendHandler(&Emac, &Emac,(XEmac_FifoHandler)emacTx_int_handler);
    XEmac_SetFifoRecvHandler(&Emac, &Emac,(XEmac_FifoHandler)emacRx_int_handler);
    XEmac_SetErrorHandler(&Emac, &Emac,(XEmac_ErrorHandler)emacErr_int_handler);

    Status = XEmac_Start(&Emac);
    if (Status != XST_SUCCESS){
		xil_printf("Failed to start EMAC\r\n");
		return XST_FAILURE;
    }
	xil_printf("complete!\r\n");
	////////////////////////////////////////////////////////

	warpphy_init();

	return 0;
}

///@brief "Good" Interrupt handler for the PHY layer.
///
///This function is the ISR for the RX physical layer peripheral.
///This function is only called if the packet has no bit errors
///(i.e., the packet passes CRCs in the PHY). The function leaves
///the payload in the receiver's buffer and notifies the user
///code via a callback, and the PHY is reset.
void rxPhyGood() {
		
	if(rxPacket->isNew==0){
		unsigned int numPayloadBytes;
		int length;

		//Copy the received packet's header into the rxPacket struct
		memcpy((unsigned char *)(&(rxPacket->header)), (unsigned char *)warpphy_getBuffAddr(controlStruct.rxBuffIndex), (size_t)NUM_HEADER_BYTES);

// OLD - byte order swapping - should be removed soon
//rxPacket->header.length = (((rxPacket->header.length)&0xFF00) >> 8) | (((rxPacket->header.length)&0x00FF) << 8);

		//Strip the header and checksum from the total number of bytes reported to user code

		if(rxPacket->header.length == NUM_HEADER_BYTES)
		{
			//Header-only packet (like an ACK) has 0 payload bytes
			rxPacket->header.length = 0;
		}
		else
		{
			//Other packets have payload bytes; subtract off header and 4-byte payload checksum lengths
			rxPacket->header.length = rxPacket->header.length - NUM_HEADER_BYTES - 4;
		}

		//If the Rx packet is too big, ignore the payload
		// This should never happen; the Tx code will never send a pkt bigger than Ethernet MTU
		if(rxPacket->header.length>MY_XEM_MAX_FRAME_SIZE){
			rxPacket->header.length = 0;
		}

		//Pass the received packet to the user handler
		usr_goodPacketHandler(rxPacket);
	}

	//Clear the good/bad packet interrupts in the PHY
	// This happens *after* the user handler returns in order to keep the PHY
	//  blocked, avoiding the case of a new Rx packet overwriting one that hasn't been processed yet
	warpphy_clearRxInterrupts();

	XIntc_Acknowledge(&Intc,XPAR_OPB_INTC_0_OFDM_TXRX_MIMO_OPBW_0_RX_INT_GOODPKT_INTR);
	return;
}

///@brief "Good Header" Interrupt handler for the PHY layer.
///
///This function is the ISR for the RX physical layer peripheral.
///This function is only called if the packet header has no bit errors
///(i.e., the packet header passes CRCs in the PHY). This interrupt will
///be asserted *before* the packet is fully received. Either the good or
///bad packet interrupt handler will be called after this.
void txPhyPktDone() {

	//Call the user's callback
	usr_txDoneHandler();

	//Clear the TxDone interrupt in the PHY
	warpphy_clearTxInterrupts();

	//Do nothing elese (for now)

	XIntc_Acknowledge(&Intc,XPAR_OPB_INTC_0_OFDM_TXRX_MIMO_OPBW_0_TX_INT_PKTDONE_INTR);
	return;
}

///@brief "Good Header" Interrupt handler for the PHY layer.
///
///This function is the ISR for the RX physical layer peripheral.
///This function is only called if the packet header has no bit errors
///(i.e., the packet header passes CRCs in the PHY). This interrupt will
///be asserted *before* the packet is fully received. Either the good or
///bad packet interrupt handler will be called after this.
void rxPhyGoodHeader() {

	//FIXME: Do we need a flag somewhere to indicate this is just a header - that the pkt is still coming?

	if(rxPacket->isNew==0)
	{
		unsigned int numPayloadBytes;
		int length;

		//Copy the received packet's header into the rxPacket struct
		memcpy((unsigned char *)(&(rxPacket->header)), (unsigned char *)warpphy_getBuffAddr(controlStruct.rxBuffIndex), (size_t)NUM_HEADER_BYTES);

		//Strip the header and checksum from the total number of bytes reported to user code
		if(rxPacket->header.length == NUM_HEADER_BYTES)
		{
			//Header-only packet (like an ACK) has 0 payload bytes
			rxPacket->header.length = 0;
		}
		else
		{
			//Other packets have payload bytes; subtract off header and 4-byte payload checksum lengths
			rxPacket->header.length = rxPacket->header.length - NUM_HEADER_BYTES - 4;
		}

		//If the Rx packet is too big, ignore the payload
		// This should never happen; the Tx code will never send a pkt bigger than Ethernet MTU
		if(rxPacket->header.length>MY_XEM_MAX_FRAME_SIZE){
			rxPacket->header.length = 0;
		}

		//Pass the received packet to the user handler
		usr_goodHeaderHandler(rxPacket);
	}

	XIntc_Acknowledge(&Intc,XPAR_OPB_INTC_0_OFDM_TXRX_MIMO_OPBW_0_RX_INT_GOODHEADER_INTR);
	return;
}

///@brief "Bad" Interrupt handler for the PHY layer.
///
///This function is the ISR for the RX physical layer peripheral.
///This function is only called if the packet fails CRC. The received
///packet is passed to the user's callback, and the PHY is reset.
void rxPhyBad(){

	//Bad packets, by definition, have no bytes which can be trusted
	// The user handler is called with no arguments, since there is no real packet data to process
	usr_badPacketHandler();

	//Clear the good/bad packet interrupts in the PHY to re-enable packet reception
	warpphy_clearRxInterrupts();

	XIntc_Acknowledge(&Intc,XPAR_OPB_INTC_0_OFDM_TXRX_MIMO_OPBW_0_RX_INT_BADPKT_INTR);
	return;
}

///@brief Handles the reception of packets from the emac core.
///
///Using the Ethernet MAC's built-in DMA, packets are copied from the XEmac
///FIFO directly into the transmit buffer of the PHY. This transmit buffer is
///specified by the user.
///@param Callback a pointer to the XEmac instance that generated the interrupt
void emacRx_int_handler(void *Callback){
	XEmac *EmacPtr = (XEmac *)Callback;
	Xuint32 length = MY_XEM_MAX_FRAME_SIZE;
	if(controlStruct.enableEthernetInt){
		XEmac_FifoRecv(EmacPtr, (void *)(warpphy_getBuffAddr(controlStruct.txBuffIndex)+NUM_HEADER_BYTES), &length);

		usr_emacHandler(length,warpphy_getBuffAddr(controlStruct.txBuffIndex)+NUM_HEADER_BYTES);
	}
	else{
	}

	XIntc_Acknowledge(&Intc,XPAR_OPB_INTC_0_ETHERNET_MAC_IP2INTC_IRPT_INTR);
	return;
}

///@brief Interrupt handler is called when the emac is finished sending a packet.
///
///This function notifies the MAC that the received wireless packet is safe
///to overwrite.
///@param Callback a pointer to the XEmac instance that generated the interrupt
void emacTx_int_handler(void *Callback){
	rxPacket->isNew=0;

	XIntc_Acknowledge(&Intc,XPAR_OPB_INTC_0_ETHERNET_MAC_IP2INTC_IRPT_INTR);
	return;
}

///@brief Interrupt handler is called if there is a problem with the emac
///
///The error code is printed to the terminal.
///@param Callback a pointer to the XEmac instance that generated the interrupt
///@param ErrCode an XStatus representing the type of error that occurred.
void emacErr_int_handler(void *Callback, XStatus ErrCode){
	xil_printf("ERROR IN EMAC #%d\r\n",ErrCode);

	XIntc_Acknowledge(&Intc,XPAR_OPB_INTC_0_ETHERNET_MAC_IP2INTC_IRPT_INTR);
	return;
}

///@brief Interrupt handler for the pushbuttons on the WARP board.
///
///This is a low-priority interrupt because it is primarily used for debugging purposes.
///The various user callbacks are executed, depending upon which button was depressed.
void pb_int_handler() {

	Xuint32 dsr;
	XStatus stat;
	char lock;

	dsr = XGpio_mGetDataReg(XPAR_PUSH_BUTTONS_4BIT_BASEADDR, 2);
	switch(dsr) {

		case 0x01:
			usr_middlebutton();
			break;

		case 0x02:
			usr_rightbutton();
			break;

		case 0x08:
			usr_upbutton();
			break;
		case 0x04:
			usr_leftbutton();
			break;

		default : {
		}

	}
	XGpio_InterruptClear(&Gpio, 0x3); // clear the interrupt

	XIntc_Acknowledge(&Intc,XPAR_OPB_INTC_0_PUSH_BUTTONS_4BIT_IP2INTC_IRPT_INTR);
	return;
}

///@brief Interrupt handler for the timer peripheral.
///
///The timer interrupt handler gets called when a hardware timer set by the
///user expires. The user's callback is called upon the completion of this event
void timer_int_handler() {
	/* Clear the timer interrupt */
	ofdm_timer_clearInterrupt();
	controlStruct.timerEnabled = 0;
	//Call User Function
	usr_timerHandler(controlStruct.timerType);
	
	XIntc_Acknowledge(&Intc,XPAR_OPB_INTC_0_OFDM_TIMER_OPBW_0_TIMEREXPIRE_INTR);
	return;
}

///@brief This function stops the timer.
///
///Additionally it will return the amount of time remaining before expiration.
///@return Number of clock cycles remaining before expiration.
int warpmac_clearTimer(){
	int time;
	time = ofdm_timer_timeLeft();
	ofdm_timer_stop();
	controlStruct.currBackoff=0;
	return time;
}

///@brief This function sets the timer to countdown for a given number of clock cycles.
///@param clocks Number of 40MHz clock cycles to countdown from
void warpmac_setTimerVal(Xuint32 clocks){
	ofdm_timer_setVal(clocks);
}

///@brief This function starts the timer in either a CSMA or non-CSMA mode.
///@param mode #ENABLECSMA if automatic carrier-sense pausing is desired. #DISABLECSMA if received energy
///is to be ignored.
void warpmac_startTimer(unsigned char mode){
	ofdm_timer_setMode(mode);
	ofdm_timer_start();
}

///@brief Generates a uniform random value between [0,N-1], where N is a power of 2
///
///Used internally by WARPMAC for random exponential backoffs.
///@param N Window size for draw of uniform random value.
int randNum(int N){
	return rand() >> (32-(N+6));
}

///@brief Returns a value corresponding to the instantaneous channel condition {busy or idle}
///
///@return 1 if medium is idle, 0 if medium is busy
unsigned int warpmac_carrierSense(){

	/* Two reasons for declaring the medium busy:
		1: The Rx PHY is actively receiving a packet.
			This only happens if packet detection and AGC already triggered, and a
			good/bad pkt interrupt will soon be asserted
		2: The packet detector indicates the average RSSI has exceeded the carrier sense
			threshold sometime in the last DIFS period
	*/
	return ofdm_pktDetector_mimo_ReadReg_pktDet_idleDifs(PKTDET_BASEADDR) && !(0x80000000 & ofdm_txrx_mimo_ReadReg_Rx_packet_done(OFDM_BASEADDR));
}

///@brief Function is responsible for high-level MAC timing
///
///This function is used by user code, and in turn calls the other timer functions.
///It is capable of initiating either a deterministic timeout, or a random backoff.
///@param type #TIMEOUT for deterministic countdown, #BACKOFF for random exponential
int warpmac_setTimer(int type){
	unsigned int setTime;
	int myRandNum;
	int retval;
	switch(type) {
		case TIMEOUT:
			controlStruct.timerType=TIMEOUT;
			setTime = controlStruct.timeout*40;
			warpmac_setTimerVal(setTime);
			warpmac_startTimer(DISABLECSMA);
			return 0;
			break;
		case BACKOFF:
			controlStruct.timerType=BACKOFF;
			myRandNum = randNum(controlStruct.currBackoff);
			setTime = myRandNum*controlStruct.slotTime*40;
			if(controlStruct.currBackoff < controlStruct.maxBackoff){
				controlStruct.currBackoff = controlStruct.currBackoff + 1;
			}
				/* Set the reset value of the timer (seconds * 40,000,000) */
				if(setTime != 0){
					warpmac_setTimerVal(setTime);
					warpmac_startTimer(ENABLECSMA);
					return 0;
				}
				if(setTime == 0){
					usr_timerHandler(controlStruct.timerType);
				}
				break;

	}
}

///@brief Pushes the given Macframe over ethernet
///@param packet Pointer to user's Macframe
void warpmac_sendEthernet(Macframe* packet){
		unsigned int length;
		length = (packet->header.length)-4;
		XEmac_FifoSend(&Emac, (void *)(warpphy_getBuffAddr(controlStruct.rxBuffIndex)+NUM_HEADER_BYTES), length);
		//rxPacket->isNew=1;
}

///@brief Attaches user callback to up button
///@param handler pointer to user's callback
void warpmac_setUpButtonHandler(void(*handler)()){
	usr_upbutton = handler;
}

///@brief Attaches user callback to left button
///@param handler pointer to user's callback
void warpmac_setLeftButtonHandler(void(*handler)()){
	usr_leftbutton = handler;
}

///@brief Attaches user callback to right button
///@param handler pointer to user's callback
void warpmac_setRightButtonHandler(void(*handler)()){
	usr_rightbutton = handler;
}

///@brief Attaches user callback to middle button
///@param handler pointer to user's callback
void warpmac_setMiddleButtonHandler(void(*handler)()){
	usr_middlebutton = handler;
}

///@brief Attaches user callback to timer
///@param handler pointer to user's callback
void warpmac_setTimerHandler(void(*handler)()){
	usr_timerHandler = handler;
}


///@brief Attaches user callback to the Emac
///@param handler pointer to user's callback
void warpmac_setEmacHandler(void(*handler)()){
	usr_emacHandler = handler;
}


///@brief Attaches user callback to PHY's Tx done interrupt
///@param handler pointer to user's callback
void warpmac_setTxDoneHandler(void(*handler)()){
	usr_txDoneHandler = handler;
}

///@brief Attaches user callback to PHY's good packet interrupt
///@param handler pointer to user's callback
void warpmac_setGoodPacketHandler(void(*handler)()){
	usr_goodPacketHandler = handler;
}

///@brief Attaches user callback to PHY's good header interrupt
///@param handler pointer to user's callback
void warpmac_setGoodHeaderHandler(void(*handler)()){
	usr_goodHeaderHandler = handler;
}

///@brief Attaches user callback to PHY's bad packet interrupt
///@param handler pointer to user's callback
void warpmac_setBadPacketHandler(void(*handler)()){
	usr_badPacketHandler = handler;
}


///@brief Alternates the bottom two LEDs on the WARP board.
void warpmac_incrementLEDLow(){
		XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, (ledStatesLow[ledStatesIndexLow]|ledStatesHigh[ledStatesIndexHigh]));
		ledStatesIndexLow = (ledStatesIndexLow+1)%2;
}


///@brief Alternates the top two LEDs on the WARP board.
void warpmac_incrementLEDHigh(){

		//xil_printf("%x\r\n", (ledStatesLow[ledStatesIndexLow]|ledStatesHigh[ledStatesIndexHigh]));
		XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, (ledStatesLow[ledStatesIndexLow]|ledStatesHigh[ledStatesIndexHigh]));
		ledStatesIndexHigh = (ledStatesIndexHigh+1)%2;
}

///@brief Sets the base rate modulation order
///
///The base rate symbols (i.e. the header) must have an agreed upon rate between the transmitter
///and receiver. This function sets that rate.
///@param rate #BPSK, #QPSK, #QAM_16, or #QAM_64
void warpmac_setBaseRate(unsigned char rate){
	warpphy_set_modulation(rate,0xF,0);
}


///@brief Sends the given Macframe via the wireless PHY.
///
///This function performs the conversion from the packet structure to the byte array
///and sends the packet over the air
///@param packet Pointer to user's Macframe
int warpmac_sendOfdm(Macframe* packet){
    int i;
    unsigned int numPayloadBytes;
    unsigned int numFullRate;
	unsigned int numBaseRate = 2;
	unsigned char sisoMode = 1;
	unsigned int numTrainingSyms = 2;
	int txResult;

	//Hold the packet detecter in reset to prevent any interfering receptions
	ofdm_pktDetector_mimo_WriteReg_pktDet_reset(PKTDET_BASEADDR, 1);

	if(packet->header.length == 0) //All-header packet, like an ACK, has only base-rate symbols
	{
		//All-header packet, like an ACK, has only base-rate symbols
		numFullRate = 0;

		//Update the PHY header length to consist of just the header
		packet->header.length = NUM_HEADER_BYTES;
	}
	else //Any other packet, that has both full- and base-rate symbols
	{
		//Other packets have both base- and full-rate data symbols
		// The number of full-rate symbols = ceil(packetLength_bytes / bytesPerOFDMSymbol)
		numFullRate = ((packet->header.length + 4 + (6*packet->header.fullRate-1))/(6*packet->header.fullRate)) * (sisoMode+1);

		//Update the PHY header length to include the MAC header bytes and full-packet checksum
		packet->header.length = NUM_HEADER_BYTES + 4 + packet->header.length;
	}

	//numSyms register has three fields:
	// bits[7:0] = number of training symbol periods (fixed)
	// bits[15:8] = number of base rate data symbols (fixed)
	// bits[23:16] = number of full rate data symbols (varries with each pkt)
	warpphy_setNumSyms(numTrainingSyms + (numBaseRate<<8) + (numFullRate<<16));

	//Copy the header to the PHY packet buffer, filling exactly NUM_HEADER_BYTES bytes
	// The rest of the packet (its payload) will be copied by DMA later
	memcpy((unsigned char*)warpphy_getBuffAddr(controlStruct.txBuffIndex), (unsigned char*) packet, (size_t)NUM_HEADER_BYTES);

	//Revert the packet payload length so the user code doesn't see the MAC/PHY tweaking
	if(packet->header.length == NUM_HEADER_BYTES)
	{
		packet->header.length = 0;
	}
	else
	{
		packet->header.length = packet->header.length - (NUM_HEADER_BYTES + 4);
	}

	//Transmit the packet; WARPPHY disables the radio Rx, enables Tx, sends the packet, then re-enables radio Rx
	// TXBLOCK will cause this call to block until the transmission is finished
	txResult = warpphy_pktTx(TXBLOCK);
	
	if(txResult != 0)
	{
		xil_printf("WARPMAC tried to Tx, but PHY was busy\r\n");
	}

	//Re-enable the packet detector so future Rx packets will be processed
	ofdm_pktDetector_mimo_WriteReg_pktDet_reset(PKTDET_BASEADDR, 0);

	return 0;
}

///@brief Tells the PHY which piece of memory to send from
///
///Also, it updates the global struct element Maccontrol#txBuffIndex to keep track of that information*/
int warpmac_setTxBuffer(unsigned int txBuff){
	controlStruct.txBuffIndex = txBuff;
	warpphy_setBuffs(controlStruct.txBuffIndex, controlStruct.rxBuffIndex);
	return 0;
}

///@brief Tells the PHY which piece of memory to receive to
///
///Also, it updates the global struct element Maccontrol#rxBuffIndex to keep track of that information*/
int warpmac_setRxBuffer(Macframe* packet, unsigned int rxBuff){
	rxPacket=packet;
	controlStruct.rxBuffIndex = rxBuff;
	warpphy_setBuffs(controlStruct.txBuffIndex, controlStruct.rxBuffIndex);
	return 0;
}

///@brief Returns which buffer PHY is set to send from
///
///@return index of PHY's buffer
int warpmac_getTxBuffer(unsigned int txBuff){
	return controlStruct.txBuffIndex;
}

///@brief Returns which buffer PHY is set to receive to
///
///@return index of PHY's buffer
int warpmac_getRxBuffer(unsigned int rxBuff){
	return (controlStruct.rxBuffIndex);
}

///@brief Increments the resend counter for the given Macframe
///
///Also, it returns whether or not the counter has wrapped around the maximum
///number of retransmits
///@param packet Pointer to user's Macframe
///@return 0 if maximum number of retransmits has been reached, and 1 otherwise
int warpmac_incrementResend(Macframe* packet){
	int status = 1;
	packet->header.currReSend = ( packet->header.currReSend+1)%((controlStruct.maxReSend)+1); //increment currReSend and wrap around max
	if( packet->header.currReSend == 0) { //if reSend wraps around, the packet will be dropped
		status = 0;
		controlStruct.currBackoff=0;
	   	packet->isNew = 0;
	}
	return status;
}

///@brief Returns a value corresponding to whether the packet was destined for this particular node.
///
///@param packet Pointer to user's Macframe
///@return 1 if destination of packet was this node, 0 otherwise
int warpmac_addressedToMe(Macframe* packet){
	int i,sum;
	unsigned char tempBoolean;
	sum = 0;
	for(i=0;i<6;i++){
		sum = sum+(packet->header.destAddr[i] == controlStruct.selfAddr[i]);
	//	xil_printf("%d ",packet->header.destAddr[i]);
	//	xil_printf("%d\r\n",controlStruct.selfAddr[i]);
	}

				tempBoolean = 0;
				if(sum==6) tempBoolean = 1;
				sum = 0;
				return tempBoolean;
}

///@brief Returns a value corresponding to whether the packet was sourced from a given node.
///
///@param packet Pointer to user's Macframe
///@param addr Pointer to char array containing MAC address to compare against
///@return 1 if destination of packet was sourced from the given address, 0 otherwise
int warpmac_addressedFromThem(Macframe* packet,unsigned char* addr){
	int i,sum;
	unsigned char tempBoolean;
	sum = 0;
	for(i=0;i<6;i++){
		sum = sum+(packet->header.srcAddr[i] == addr[i]);
				}
				tempBoolean = 0;
				if(sum==6) tempBoolean = 1;
				sum = 0;

				return tempBoolean;
}

///@brief Returns a value corresponding to whether the packet was destined for a given node.
///
///@param packet Pointer to user's Macframe
///@param addr Pointer to char array containing MAC address to compare against
///@return 1 if destination of packet was sourced from the given address, 0 otherwise
int warpmac_addressedToThem(Macframe* packet,unsigned char* addr){
	int i,sum;
	unsigned char tempBoolean;
	sum = 0;
	for(i=0;i<6;i++){
		sum = sum+(packet->header.destAddr[i] == addr[i]);
				}
				tempBoolean = 0;
				if(sum==6) tempBoolean = 1;
				sum = 0;
				return tempBoolean;
}

///@brief Sets the maximum number of resends
///
///@param c Integer maximum number of resends
void warpmac_setMaxResend(unsigned int c){
	controlStruct.maxReSend=c;
}

///@brief Sets the maximum contention window.
///
///@param c Maximum contention window
void warpmac_setMaxCW(unsigned int c){
	controlStruct.maxBackoff=c;
}

///@brief Sets wireless MAC address of this node.
///
///@param addr Pointer to char array containing MAC address
int warpmac_setMacAddr(unsigned char* addr){
	controlStruct.selfAddr[5] = addr[5];
	controlStruct.selfAddr[4] = addr[4];
	controlStruct.selfAddr[3] = addr[3];
	controlStruct.selfAddr[2] = addr[2];
	controlStruct.selfAddr[1] = addr[1];
	controlStruct.selfAddr[0] = addr[0];
}

///@brief Sets the amount of time the node is willing to wait for an acknowledgement.
///
///@param time Timeout duration (in microseconds)
int warpmac_setTimeout(unsigned int time){
	controlStruct.timeout = time;
}

///@brief Sets the smallest backoff window.
///
///@param time Slot time duration (in microseconds)
int warpmac_setSlotTime(unsigned int time){
	controlStruct.slotTime = time;
}

///@brief Returns a value corresponding to whether or not the node is in timeout.
///
///@return 1 if in #TIMEOUT, 0 otherwise
int warpmac_inTimeout(){
	return controlStruct.timerType==TIMEOUT;
}

///@brief Reads the value from the user dip switches for use as node identification.
///
///@return Value currently set on dip switches of the WARP board
int warpmac_getMyId(){
	int ret;
	ret =  XGpio_mGetDataReg(XPAR_DIPSWS_4BIT_BASEADDR,1);
	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1, sevenSegmentMap(ret));
	//This function returns the value currently specified in the Dip Switch on the FPGA board.
	return ret;
}

///@brief Lowers carrier sense threshold such that CSMA state is enabled
void warpmac_enableCSMA(){
	ofdm_pktDetector_mimo_WriteReg_csma_avgThresh(PKTDET_BASEADDR, 4000);
	return;
}

///@brief Raises carrier sense threshold such that CSMA state is disabled
void warpmac_disableCSMA(){
	ofdm_pktDetector_mimo_WriteReg_csma_avgThresh(PKTDET_BASEADDR, 16000);
	return;
}

///@brief Maps character to the seven segment display
///
///@param x Input character
///@return Corresponding value that can be written to the GPIO connected to the Hex displays
unsigned char sevenSegmentMap(unsigned char x){
	switch(x)
	{
		case(0x0) : return 0x007E;
		case(0x1) : return 0x0030;
		case(0x2) : return 0x006D;
		case(0x3) : return 0x0079;
		case(0x4) : return 0x0033;
		case(0x5) : return 0x005B;
		case(0x6) : return 0x005F;
		case(0x7) : return 0x0070;
		case(0x8) : return 0x007F;
		case(0x9) : return 0x007B;

		case(0xA) : return 0x0077;
		case(0xB) : return 0x007F;
		case(0xC) : return 0x004E;
		case(0xD) : return 0x007E;
		case(0xE) : return 0x004F;
		case(0xF) : return 0x0047;
		default : return 0x0000;
	}
}

///@brief Displays a single bit on the left hex display
///
///@param x Character to display
void warpmac_leftHexBit(unsigned char x){
	//XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1, sevenSegmentMap(x));
	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1, 0x1<<x);
}

///@brief Displays a single bit on the right hex display
///
///@param x Character to display
void warpmac_rightHexBit(unsigned char x){
	//XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1, sevenSegmentMap(x));
	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1, 0x1<<x);
}

///@brief Displays the input character on the left hex display
///
///@param x Character to display
void warpmac_leftHex(unsigned char x){
	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1, sevenSegmentMap(x));
//	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_1_BASEADDR, 1, 0x1<<x);

	return;
}

///@brief Displays the input character on the right hex display
///
///@param x Character to display
void warpmac_rightHex(unsigned char x){
	XGpio_mSetDataReg(XPAR_LED_7SEGMENT_BASEADDR, 1, sevenSegmentMap(x));

	return;
}

///@brief Raises a signal that is routed out to the debug header on the board
///
///@param val 4-bit input that toggles
void warpmac_setDebugGPIO(unsigned char val){
	XGpio_mSetDataReg(XPAR_DEBUGOUTPUTS_BASEADDR, 1, val);

	return;
}

///@brief Enabled the Ethernet Interrupt
void warpmac_enableEthernetInterrupt(){
	controlStruct.enableEthernetInt=1;
	XIntc_Enable(&Intc, XPAR_OPB_INTC_0_ETHERNET_MAC_IP2INTC_IRPT_INTR);

	return;
}




///@brief Disables the Ethernet Interrupt
void warpmac_disableEthernetInterrupt(){
	XIntc_Disable(&Intc, XPAR_OPB_INTC_0_ETHERNET_MAC_IP2INTC_IRPT_INTR);
	controlStruct.enableEthernetInt=0;
	
	return;
}
